/* --COPYRIGHT--,BSD
 * Copyright (c) 2013, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/* 
 * ======== HAL_SDCard.c ========
 *
 * Nessim Ben Ammar 29.08.2014
 * Removed SD card relevant bits / pins. These are defined in the HAL.h instead.
 * Changed all functions to use the driverlib and created the abstraction:
 * The functions where bounded to USCI_B1 and a specific Master clock of 25 MHz
 * The functions has been changed to call the clock references from HAL.h
 *
 */

/***************************************************************************//**
 * @file       HAL_SDCard.c
 * @addtogroup HAL_SDCard
 * @{
 ******************************************************************************/
#include "msp430.h"
#include "HAL_SDCard.h"
#include "driverlib.h"

/***************************************************************************//**
 * @brief   Initialize SD Card
 * @param   None
 * @return  None
 ******************************************************************************/
void SDCard_init(void)
{
	// Port initialization for SD Card operation
	// SPI SIMO and SOMI are pulled up.
	GPIO_setAsInputPinWithPullUpResistor(SD_SPI_PORT,
			SD_SPI_SOMI | SD_SPI_SIMO);
	GPIO_setAsPeripheralModuleFunctionOutputPin(SD_SPI_PORT,
			SD_SPI_SIMO | SD_SPI_CLK);
	GPIO_setAsPeripheralModuleFunctionInputPin(SD_SPI_PORT, SD_SPI_SOMI);
	GPIO_setOutputHighOnPin(SD_CS_PORT, SD_CS_PIN);
	GPIO_setAsOutputPin(SD_CS_PORT, SD_CS_PIN);
	// Clock must be < 400 kHz at initialization.
	USCI_SPI_masterInit(SD_SPI_BASE, USCI_SPI_CLOCKSOURCE_SMCLK, SYS_CLK, 390000,
			USCI_SPI_MSB_FIRST, USCI_SPI_PHASE_DATA_CHANGED_ONFIRST_CAPTURED_ON_NEXT,
			USCI_SPI_CLOCKPOLARITY_INACTIVITY_HIGH);
	USCI_SPI_enable(SD_SPI_BASE);
	USCI_SPI_clearInterruptFlag(SD_SPI_BASE, USCI_SPI_RECEIVE_INTERRUPT);
}

/***************************************************************************//**
 * @brief   Enable fast SD Card SPI transfers. This function is typically
 *          called after the initial SD Card setup is done to maximize
 *          transfer speed.
 * @param   None
 * @return  None
 ******************************************************************************/
void SDCard_fastMode(void)
{
	USCI_SPI_changeMasterClockParam fastMode;
	fastMode.clockSourceFrequency = SYS_CLK;
	fastMode.desiredSpiClock = 12500000;			// Change SPI CLK freq. to 12.5 MHz
	USCI_SPI_changeMasterClock(SD_SPI_BASE, & fastMode);
}

/***************************************************************************//**
 * @brief   Read a frame of bytes via SPI
 * @param   pBuffer Place to store the received bytes
 * @param   size Indicator of how many bytes to receive
 * @return  None
 ******************************************************************************/
void SDCard_readFrame(uint8_t *pBuffer, uint16_t size)
{
	uint16_t gie = __get_SR_register() & GIE;            //Store current GIE state

	__disable_interrupt();                            //Make this operation atomic
	//Ensure RXIFG is clear
	USCI_SPI_clearInterruptFlag(SD_SPI_BASE, USCI_SPI_RECEIVE_INTERRUPT);
	//Clock the actual data transfer and receive the bytes
	while (size--)
	{
		while ( ! USCI_SPI_getInterruptStatus(SD_SPI_BASE,
				USCI_SPI_TRANSMIT_INTERRUPT))
			;                      //Wait while not ready for TX
		USCI_SPI_transmitData(SD_SPI_BASE, 0xff);                 //Write dummy byte
		while ( ! USCI_SPI_getInterruptStatus(SD_SPI_BASE,
				USCI_SPI_RECEIVE_INTERRUPT))
			;                      //Wait for RX buffer (full)
		* pBuffer++ = USCI_SPI_receiveData(SD_SPI_BASE);
	}

	__bis_SR_register(gie);
	//Restore original GIE state
}

/***************************************************************************//**
 * @brief   Send a frame of bytes via SPI
 * @param   pBuffer Place that holds the bytes to send
 * @param   size Indicator of how many bytes to send
 * @return  None
 ******************************************************************************/
void SDCard_sendFrame(uint8_t *pBuffer, uint16_t size)
{
	uint16_t gie = __get_SR_register() & GIE;            //Store current GIE state

	__disable_interrupt();                            //Make this operation atomic

	//Clock the actual data transfer and send the bytes. Note that we
	//intentionally not read out the receive buffer during frame transmission
	//in order to optimize transfer speed, however we need to take care of the
	//resulting overrun condition.
	while (size--)
	{
		while ( ! USCI_SPI_getInterruptStatus(SD_SPI_BASE,
				USCI_SPI_TRANSMIT_INTERRUPT))
			;                      //Wait while not ready for TX
		USCI_SPI_transmitData(SD_SPI_BASE, *pBuffer++);                  //Write byte
	}
	while (USCI_SPI_isBusy(SD_SPI_BASE))
		;                             //Wait for all TX/RX to finish

	USCI_SPI_receiveData(SD_SPI_BASE);          //Dummy read to empty RX buffer
																							//and clear any overrun conditions

	__bis_SR_register(gie);
	//Restore original GIE state
}

/***************************************************************************//**
 * @brief   Set the SD Card's chip-select signal to high
 * @param   None
 * @return  None
 ******************************************************************************/
void SDCard_setCSHigh(void)
{
	GPIO_setOutputHighOnPin(SD_CS_PORT, SD_CS_PIN);
}

/***************************************************************************//**
 * @brief   Set the SD Card's chip-select signal to low
 * @param   None
 * @return  None
 ******************************************************************************/
void SDCard_setCSLow(void)
{
	GPIO_setOutputLowOnPin(SD_CS_PORT, SD_CS_PIN);
}

/***************************************************************************//**
 * @}
 ******************************************************************************/
//Released_Version_4_10_00
