/*
 *  Methods implementation of the software library for SD-Cards.
 * 	To be used with one SD-Card connected to the MCU via SPI.
 * 	Do not use this library with USB Mass Storage Class!
 *
 * 	This library relies on the driverlib of Ti 1.90.00.65
 * 	This driverlib is to be used with the MSP430F5xxx_6xxx family
 *  Last Revision on: 29.08.2014
 *  Version: 0.9.00
 *  Copyright 2014, Micronas GmbH.
 *  Author: Nessim Ben Ammar
 */

#include "SD_Card.h"

FATFS SD_FileSystemObject;
FIL SD_FileObject;
FRESULT SD_OperationResult;
volatile uint8_t SD_EREG;
volatile uint8_t SD_AREG;

// ________________________________________________________________________________________________

/*
 * This method is used initialize the SD CARD detection system.
 * This method will call SD_DetectCard() if a card is present:
 * If a SD_INS_PIN & SD_INS_PORT are defined
 * 	This method will set up the interrupt vector, so insertion
 * 	or removal will be automatically updated.
 *
 * Otherwise a timer interrupt will trigger the card detection.
 *
 * Its is strongly recommended to call this method once at the beginning
 * of the program, and then to use SD_DetectCard() to check for card presence.
 *
 * Modifies:
 *  - SD_INS_PORT / SD_INS_PIN.
 *
 */

void SD_Init()
{
#if defined (SD_INS_PORT) && defined (SD_INS_PIN)
	GPIO_setAsInputPinWithPullDownResistor(SD_INS_PORT, SD_INS_PIN);
	if (GPIO_getInputPinValue(SD_INS_PORT, SD_INS_PIN))
	{
		// SD Card is inserted.
		// Program card removal interrupt edge.
		GPIO_setAsInputPin(SD_INS_PORT, SD_INS_PIN);
		GPIO_interruptEdgeSelect(SD_INS_PORT, SD_INS_PIN,
				GPIO_HIGH_TO_LOW_TRANSITION);
		GPIO_enableInterrupt(SD_INS_PORT, SD_INS_PIN);
		// Proceed detecting Card.
		SD_DetectCard();
	}
	else
	{
		// No SD Card is inserted.
		// Program card insertion interrupt edge.
		GPIO_interruptEdgeSelect(SD_INS_PORT, SD_INS_PIN,
				GPIO_LOW_TO_HIGH_TRANSITION);
		GPIO_enableInterrupt(SD_INS_PORT, SD_INS_PIN);
	}
#else
	// Detect card
	SD_DetectCard();
	// Schedule a timer interrupt after to detect insertion or removal every 5 seconds.
	TIMER_A_startUpMode(SD_TIMER_BASE, TIMER_A_CLOCKSOURCE_ACLK,
			TIMER_A_CLOCKSOURCE_DIVIDER_32, 5000 + 1, TIMER_A_TAIE_INTERRUPT_DISABLE,
			TIMER_A_CCIE_CCR0_INTERRUPT_ENABLE, TIMER_A_DO_CLEAR);
#endif
}

// ________________________________________________________________________________________________

/*
 * This method is used to detect the presence of an SD Card.
 * Not necessary but strongly suggested, to have SD_Init() called once
 * before this method. See the description of SD_Init().
 *
 * Guard:
 *  - No error flags pending except card insertion error.
 *
 * Modifies:
 *  - SD SPI / CS pins & ports.
 *  - SD_AREG
 *
 */

void SD_DetectCard()
{
	if (SD_EREG & ~ SD_EFLAG_INSRT) return;
	DSTATUS init;
	// initialize the disk
	init = disk_initialize(0);
	if (init == RES_OK)
	{
		// Card seems to be present.
		if (detectCard())
		{
			// Card is definitely operative.
			SD_AREG |= SD_AFLAG_INSRT;
			SD_EREG &= ~ SD_EFLAG_INSRT;
			return;
		}
	}
	SD_AREG &= ~ SD_AFLAG_INSRT;
	SD_EREG |= SD_EFLAG_INSRT;
}

// ________________________________________________________________________________________________

/*
 * This method is used to mount the file system of the SD Card.
 * The file system must be mounted before any operation can take place.
 *
 * Guard:
 *  - Card must be inserted (detected).
 *  - No error flags pending except mounting FS error.
 *
 * Modifies:
 *  - SD_FileSystemObject
 *  - SD_OperationResult
 *  - SD_AREG
 *
 */

void SD_MountFS()
{
	if (SD_EREG & ~ SD_EFLAG_FSMNT) return;
	if ( ~ SD_AREG & SD_AFLAG_INSRT)
	{
		// SD card is not inserted, set error and exit.
		SD_EREG |= SD_EFLAG_INSRT;
		return;
	}
	// Mount file system.
	SD_OperationResult = f_mount(0, & SD_FileSystemObject);
	if (SD_OperationResult == FR_OK)
	{
		// File system mounted successfully.
		SD_AREG |= SD_AFLAG_FSMNT;
		SD_EREG &= ~ SD_EFLAG_FSMNT;
	}
	else
	{
		// Failed, set error and exit.
		SD_AREG &= ~ SD_AFLAG_FSMNT;
		SD_EREG |= SD_EFLAG_FSMNT;
		return;
	}
}

// ________________________________________________________________________________________________

/*
 * This method is used to unmount the file system of the SD Card.
 * The file system should be unmounted when done with the SD Card.
 *
 * Modifies:
 *  - SD_FileSystemObject
 *  - SD_OperationResult
 *  - SD_AREG
 *
 */

void SD_UnmountFS()
{
	// Unmount file system.
	SD_OperationResult = f_mount(0, NULL);
	if (SD_OperationResult == FR_OK)
	{
		/// File system unmounted successfully.
		SD_AREG &= ~ SD_AFLAG_FSMNT;
	}
	else
	{
		// Failed, set error and exit.
		SD_EREG |= SD_EFLAG_FSMNT;
		return;
	}
}

// ________________________________________________________________________________________________

/*
 * This function creates a directory with the given name or path.
 * The directory name must be 8 or less characters long.
 * Only one directory is created at a time:
 * Example:
 * 	/sub1/sub2/sub3, must be done in 3 steps:
 * 	SD_MkDir("/sub1"); SD_MkDir("/sub1/sub2"); SD_MkDir("/sub1/sub2/sub3");
 * This function will not trigger an error if the directory already exists.
 *
 * Guard:
 *  - Card must be inserted (detected).
 *  - File system must be mounted.
 *  - No error flags pending.
 *
 * Valid parameters:
 * 	- a valid directory path (name of folder <= 8 characters).
 * 	- or a valid directory name (name of folder <= 8 characters).
 *
 */

void SD_MkDir(char * dirName)
{
	if ( ~ SD_AREG & SD_AFLAG_INSRT)
	{
		// SD card is not inserted, set error and exit.
		SD_EREG |= SD_EFLAG_INSRT;
		return;
	}
	if ( ~ SD_AREG & SD_EFLAG_FSMNT)
	{
		// File system is not mounted, set error and exit.
		SD_EREG |= SD_EFLAG_FSMNT;
		return;
	}
	// If any error pending, exit the function.
	if (SD_EREG) return;
	// Create the directory
	SD_OperationResult = f_mkdir(dirName);
	// If operation succeeded, exit the function.
	if (SD_OperationResult == FR_OK || SD_OperationResult == FR_EXIST) return;
	// An error occurred, set  corresponding error flag.
	SD_EREG |= SD_EFLAG_MKDIR;
}

// ________________________________________________________________________________________________

/*
 * This function creates a file with the given name or path.
 * The file name must be 8.3 or less characters long.
 * Only one file can be open at a time:
 * Example:
 * 	-SD_OpenFile("/sub1/message.txt");
 * 	Opens a file named message.txt in the sub-directory sub1.
 * If the file already exists, it will be overwritten.
 *
 * Guard:
 *  - Card must be inserted (detected).
 *  - File system must be mounted.
 *  - No file is open at the moment.
 *  - No error flags pending.
 *
 * Valid parameters:
 * 	- a valid file path (name of file <= 8.3 characters).
 * 	- or a valid file name (name of file <= 8.3 characters).
 *
 * Modifies:
 *  - SD_FileObject
 *  - SD_OperationResult
 *  - SD_AREG
 *
 */

void SD_OpentFile(char *fileName)
{
	// If case of any error, exit the function.
	if (SD_EREG & ~ SD_EFLAG_FIOPN) return;
	if ( ~ SD_AREG & SD_AFLAG_INSRT)
	{
		// SD card is not inserted, set error and exit.
		SD_EREG |= SD_EFLAG_INSRT;
		return;
	}
	if ( ~ SD_AREG & SD_EFLAG_FSMNT)
	{
		// File system is not mounted, set error and exit.
		SD_EREG |= SD_EFLAG_FSMNT;
		return;
	}
	if (SD_AREG & SD_AFLAG_FIOPN)
	{
		// A file is already opened, set error and exit.
		SD_EREG |= SD_EFLAG_FIOPN;
	}
	// Open file
	SD_OperationResult = f_open( & SD_FileObject, fileName,
			FA_CREATE_ALWAYS + FA_WRITE);
	if (SD_OperationResult == FR_OK)
	{
		// File opened successfully.
		SD_AREG |= SD_AFLAG_FIOPN;
		SD_EREG &= ~ SD_EFLAG_FIOPN;
	}
	else
	{
		// Failed, set error and exit.
		SD_AREG &= ~ SD_AFLAG_FIOPN;
		SD_EREG |= SD_EFLAG_FIOPN;
		return;
	}
}

// ________________________________________________________________________________________________

/*
 * This function writes the given byte sequence to the currently opened file.
 *
 * Guard:
 *  - Card must be inserted (detected).
 *  - File system must be mounted.
 *  - File must be already opened.
 *  - No error flags pending.
 *
 * Valid parameters:
 * 	- a null terminated string.
 *
 */

void SD_WriteToFile(char * entry)
{
	// If case of any error, exit the function.
	if (SD_EREG) return;
	if ( ~ SD_AREG & SD_AFLAG_INSRT)
	{
		// SD card is not inserted, set error and exit.
		SD_EREG |= SD_EFLAG_INSRT;
		return;
	}
	if ( ~ SD_AREG & SD_EFLAG_FSMNT)
	{
		// File system is not mounted, set error and exit.
		SD_EREG |= SD_EFLAG_FSMNT;
		return;
	}
	if ( ~ SD_AREG & SD_AFLAG_FIOPN)
	{
		// File is not opened, set error and exit.
		SD_EREG |= SD_EFLAG_FIOPN;
		return;
	}
	// Write byte sequence.
	uint16_t numOfWrittenBytes;
	SD_OperationResult = f_write( & SD_FileObject, entry, strlen(entry),
			& numOfWrittenBytes);
	if ((numOfWrittenBytes == strlen(entry)) && (SD_OperationResult == FR_OK))
	{
		// Entry was written successfully.
		// Synchronize data (data will be physically written now).
		SD_OperationResult = f_sync( & SD_FileObject);
		// If no errors have been detected, the sequence is physically written.
		// In that case exit the function.
		if (SD_OperationResult == FR_OK) return;
	}
	// Entry has not been written properly
	SD_EREG |= SD_EFLAG_FIWRT;
}

// ________________________________________________________________________________________________

/*
 * This function closes the currently opened file
 *
 * Modifies:
 *  - SD_FileObject
 *  - SD_OperationResult
 *  - SD_AREG
 *
 */

void SD_CloseFile()
{
	// Close file object.
	SD_OperationResult = f_close( & SD_FileObject);
	if (SD_OperationResult == FR_OK)
	{
		SD_AREG &= ~ SD_AFLAG_FIOPN;
		return;
	}
	// File did not close properly
	SD_EREG |= SD_EFLAG_FICLS;
}

// ________________________________________________________________________________________________

/*
 * This function clears all error flags.
 *
 * Modifies:
 *  - SD_EREG
 *
 */

void SD_ClearErrors()
{
	SD_EREG = 0;
}
