/*
 *  Interrupt service routines implementation of the software library for SD Cards.
 * 	To be used with one SD Card connected to the MCU.
 *
 * 	This library relies on the driverlib of Ti 1.90.00.65
 * 	This driverlib is to be used with the MSP430F5xxx_6xxx family
 *  Last Revision on: 29.08.2014
 *  Version: 0.9.00
 *  Copyright 2014, Micronas GmbH.
 *  Author: Nessim Ben Ammar
 */

#include "SD_Card.h"

// ________________________________________________________________________________________________

/*
 * SD_PLUG port interrupt
 *
 */

#if defined(SD_INS_PORT) && defined (SD_INS_PIN)
// Define the interrupt vector corresponding to SD_INS_PORT.
#if (SD_INS_PORT == GPIO_PORT_P1)
#define SD_INS_VECTOR PORT1_VECTOR
#endif
#if SD_INS_PORT == GPIO_PORT_P2
#define SD_INS_VECTOR PORT2_VECTOR
#endif

#pragma vector = SD_INS_VECTOR
__interrupt void SD_PLUG_ISR()
{
	// Set pull down to pump out floating currents on the insertion pin .
	GPIO_setAsInputPinWithPullDownResistor(SD_INS_PORT, SD_INS_PIN);
	// Schedule a timer interrupt after one second to detect the card.
	TIMER_A_startUpMode(SD_TIMER_BASE, TIMER_A_CLOCKSOURCE_ACLK,
			TIMER_A_CLOCKSOURCE_DIVIDER_32, 1000 + 1, TIMER_A_TAIE_INTERRUPT_DISABLE,
			TIMER_A_CCIE_CCR0_INTERRUPT_ENABLE, TIMER_A_DO_CLEAR);
	// Clear the interrupt flag.
	GPIO_clearInterruptFlag(SD_INS_PORT, SD_INS_PIN);
}
#endif

// ________________________________________________________________________________________________

/*
 * Timer A interrupt
 *
 */

#if SD_TIMER_BASE == TIMER_A0_BASE
#define TIMER_VECTOR	TIMER0_A0_VECTOR
#endif
#if SD_TIMER_BASE == TIMER_A1_BASE
#define TIMER_VECTOR	TIMER1_A0_VECTOR
#endif
#if SD_TIMER_BASE == TIMER_A2_BASE
#define TIMER_VECTOR	TIMER2_A0_VECTOR
#endif

#pragma vector = TIMER_VECTOR
__interrupt void TIMER_ISR()
{
	// This part is triggered by the SD_INS_PIN interrupt.
	// If the insert pin is not used, this part will not be compiled.
#if defined(SD_INS_PORT) && defined (SD_INS_PIN)
	// Stop timer to avoid other interrupts.
	TIMER_A_stop(SD_TIMER_BASE);
	// Proceed to update the SD card state.
	if (GPIO_getInputPinValue(SD_INS_PORT, SD_INS_PIN))
	{
		// SD Card is inserted.
		// Program card removal interrupt edge.
		GPIO_setAsInputPin(SD_INS_PORT, SD_INS_PIN);
		GPIO_interruptEdgeSelect(SD_INS_PORT, SD_INS_PIN,
				GPIO_HIGH_TO_LOW_TRANSITION);
		GPIO_enableInterrupt(SD_INS_PORT, SD_INS_PIN);
	}
	else
	{
		// No SD Card is inserted / has been removed.
		// Program card insertion interrupt edge.
		GPIO_interruptEdgeSelect(SD_INS_PORT, SD_INS_PIN,
				GPIO_LOW_TO_HIGH_TRANSITION);
		GPIO_enableInterrupt(SD_INS_PORT, SD_INS_PIN);
	}
#endif
	SD_DetectCard();
}

